
from django.contrib import admin

from diplomat.models import ISOCountry, ISOLanguage

class ISOCountryAdmin(admin.ModelAdmin):
	list_display  = ('alpha2', 'alpha3', 'numeric', 'name', 'official_name')
	ordering      = ('name',)
	search_fields = ('name', 'official_name')

class ISOLanguageAdmin(admin.ModelAdmin):
	list_display  = ('alpha2', 'bibliographic', 'terminology',  'name')
	ordering      = ('name',)
	search_fields = ('name',)

admin.site.register(ISOCountry, ISOCountryAdmin)
admin.site.register(ISOLanguage, ISOLanguageAdmin)




import re

from autoslug import AutoSlugField
from django.db import models
from django.utils.translation import ugettext_lazy as _
import pycountry

class PycountryManager(models.Manager):
	"""Base manager for any model that shadows a pycountry database."""

	def sync(self):
		""""Syncs the managed model with its pycountry equivalent."""

		model_fields = self.model._meta.get_all_field_names()

		# Clear all existing model instances
		self.all().delete()

		# Create a new model instance for every datum in the pycountry database
		for resource in self.provide_pycountry_database():
			kwargs = {}
			for field_name in model_fields:
				if hasattr(resource, field_name):
					kwargs[field_name] = getattr(resource, field_name)
			self.create(**kwargs)

	def provide_pycountry_database(self):
		"""Provide the pycountry database that the model shadows."""
		raise NotImplementedError

class ISOCountryManager(models.Manager):
	"""Custom manager for the ISOCountry model."""

	def get_by_natural_key(self, alpha3):
		return self.get(alpha3=alpha3)

class ISOCountryPycountryManager(PycountryManager):
	"""Pycountry manager for the ISOCountry model."""

	def provide_pycountry_database(self):
		return pycountry.countries

class ISOCountry(models.Model):
	"""A country covered by the ISO 3166-1 standard."""

	objects   = ISOCountryManager()
	pycountry = ISOCountryPycountryManager()

	alpha2  	  = models.CharField(max_length=2, verbose_name=_("alpha-2 code"), unique=True)
	alpha3  	  = models.CharField(max_length=3, verbose_name=_("alpha-3 code"), unique=True)
	numeric 	  = models.CharField(max_length=3, verbose_name=_("numeric code"))
	name          = models.CharField(max_length=75, verbose_name=_("name"), unique=True)
	slug          = AutoSlugField(max_length=75, verbose_name=_("slug"), populate_from="name", unique=True)
	official_name = models.CharField(max_length=75, verbose_name=_("official name"), null=True, blank=True)

	class Meta:
		ordering = ('name',)
		verbose_name = _("ISO 3166 country")
		verbose_name_plural = _("ISO 3166 countries")

	def __unicode__(self):
		return self.name

	def natural_key(self):
		return (self.alpha3,)

class ISOLanguageManager(models.Manager):
	"""Custom manager for the ISOLanguage model."""

	# The special non-language ISO 639-2 codes
	_NON_LANGUAGES_CODES = ('mis', 'mul', 'qaa-qtz', 'und', 'zxx')

	def basic(self):
		"""Return a queryset of languages covered by the ISO 639-1 standard."""
		return self.filter(alpha2__isnull=False)

	def usable(self):
		"""The full list of usable ISO 639-2 human languages.

		This is effectively a list of all languages with the special ISO 639-2
		non-language codes excluded.
		"""
		return self.all().exclude(terminology__in=self._NON_LANGUAGES_CODES)

	def get_by_natural_key(self, terminology):
		return self.get(terminology=terminology)

class ISOLanguagePycountryManager(PycountryManager):
	"""Pycountry manager for the ISOLanguage model."""

	def provide_pycountry_database(self):
		return pycountry.languages

class ISOLanguage(models.Model):
	"""A language covered by the ISO 639-2 standard."""

	objects   = ISOLanguageManager()
	pycountry = ISOLanguagePycountryManager()

	alpha2        = models.CharField(max_length=2, verbose_name=_("ISO 639-1 identifier"), null=True, blank=True)
	bibliographic = models.CharField(max_length=7, verbose_name=_("ISO 639-2 bibliographic identifier"), unique=True)
	terminology   = models.CharField(max_length=7, verbose_name=_("ISO 639-2 terminology identifier"), unique=True)
	name          = models.CharField(max_length=100, verbose_name=_("name"), unique=True)
	slug          = AutoSlugField(max_length=100, verbose_name=_("name slug"), populate_from="name", unique=True)

	class Meta:
		ordering = ('name',)
		verbose_name = _("ISO 639 language")
		verbose_name_plural = _("ISO 639 languages")

	def __unicode__(self):
		return self.name

	def natural_key(self):
		return (self.terminology,)

	@property
	def simple_name(self):
		"""The language name stripped of variant spellings and date ranges."""
		return re.split(r'[;\(]', self.name)[0].strip()




from django.core.management.base import BaseCommand
from django.utils.translation import ugettext as _

from diplomat.models import ISOCountry, ISOLanguage

class Command(BaseCommand):

	help = _("creates and populates language and country models")

	def handle(self, *args, **kwargs):
		"""Create a Django database mirroring the pycountry database."""

		print _("Creating country list...")
		ISOCountry.pycountry.sync()

		print _("Creating language list...")
		ISOLanguage.pycountry.sync()




from django.test import TestCase
import pycountry

from diplomat.models import ISOCountry, ISOLanguage

class ISOCountryTest(TestCase):
	"""Tests for the ISOCountry model."""

	def test_sync(self):
		"""The ISOCountry instances should exactly match the pycountry countries."""
		ISOCountry.pycountry.sync()
		for country in pycountry.countries:
			isocountry = ISOCountry.objects.get(alpha2=country.alpha2)
			for attr in ['alpha2', 'alpha3', 'numeric', 'name', 'official_name']:
				if hasattr(country, attr):
					self.assertEqual(getattr(country, attr), getattr(isocountry, attr))

class ISOLanguageTest(TestCase):
	"""Tests for the ISOLanguage model."""

	def test_sync(self):
		"""The ISOCLanguage instances should exactly match the pycountry languages."""
		ISOLanguage.pycountry.sync()
		for language in pycountry.languages:
			isolanguage = ISOLanguage.objects.get(terminology=language.terminology)
			for attr in ['alpha2', 'bibliographic', 'terminology', 'name']:
				if hasattr(language, attr):
					self.assertEqual(getattr(language, attr), getattr(isolanguage, attr))

	def test_basic_languages(self):
		"""A queryset of basic languages covered by the ISO 639-1 standard should be obtainable."""
		self.assertFalse(ISOLanguage.objects.basic().filter(alpha2__isnull=True).count())

	def test_usable_languages(self):
		"""A queryset of all usable languages covered by the ISO 639-2 standard should be obtainable."""
		non_language_codes = ['mis', 'mul', 'qaa', 'und', 'zxx']
		usable_languages = ISOLanguage.objects.usable()
		for code in non_language_codes:
			self.assertFalse(usable_languages.filter(terminology__startswith=code).count())

	def test_queryset_lengths(self):
		"""The queryset size order, from largest to smallest, is all, usable, basic."""
		self.assertGreater(ISOLanguage.objects.all().count(), ISOLanguage.objects.usable().count())
		self.assertGreater(ISOLanguage.objects.usable().count(), ISOLanguage.objects.basic().count())

	def test_simple_name(self):
		"""The simple name of a language should omit any alternative spellings or date ranges."""
		old_english = ISOLanguage.objects.get(terminology='ang')
		adyghe = ISOLanguage.objects.get(terminology='ady')
		self.assertEqual('English, Old', old_english.simple_name)
		self.assertEqual('Adyghe', adyghe.simple_name)




from setuptools import find_packages, setup

setup(
	name='django-diplomat',
	version=__import__('diplomat').__version__,
	description='Language and country models for Django derived from the pycountry module.',
	author='Justin Locsei',
	author_email='justin.locsei@oberlin.edu',
	url='http://github.com/oberlin/django-diplomat/',
	download_url='https://github.com/oberlin/django-diplomat/zipball/master',
	long_description=open('README.md', 'r').read(),
	packages=find_packages(),
	install_requires=[
		'django-autoslug',
		'pycountry'
	],
	include_package_data=True,
	license="BSD",
	platforms='any',
	zip_safe=False,
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Environment :: Web Environment',
		'Framework :: Django',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: BSD License',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Topic :: Software Development :: Libraries :: Python Modules'
	]
)




from django import forms

from diplomat.models import ISOCountry, ISOLanguage

class _CustomChoiceField(object):
	"""Base class for a custom choice field that can provide a default queryset."""

	def __init__(self, *args, **kwargs):
		kwargs['queryset'] = self.provide_queryset()
		super(_CustomChoiceField, self).__init__(*args, **kwargs)

	def provide_queryset(self):
		"""Allow a child field to override the default queryset of the choice field."""
		raise NotImplementedError

class _LanguageField(_CustomChoiceField):
	"""Base class for any field for selecting a language."""

	def label_from_instance(self, language):
		"""Display the language's simple name."""
		return language.simple_name

class LanguageChoiceField(_LanguageField, forms.ModelChoiceField):
	"""A field for choosing a single language."""
	def provide_queryset(self):
		return ISOLanguage.objects.usable()

class LanguageMultipleChoiceField(_LanguageField, forms.ModelMultipleChoiceField):
	"""A field for choosing multiple languages."""
	def provide_queryset(self):
		return ISOLanguage.objects.usable()

class BasicLanguageChoiceField(_LanguageField, forms.ModelChoiceField):
	"""A select field for choosing a single basic language."""
	def provide_queryset(self):
		return ISOLanguage.objects.basic()

class BasicLanguageMultipleChoiceField(_LanguageField, forms.ModelMultipleChoiceField):
	"""A select field for choosing multiple basic languages."""
	def provide_queryset(self):
		return ISOLanguage.objects.basic()

class CountryChoiceField(_CustomChoiceField, forms.ModelChoiceField):
	"""A field for choosing a single country."""
	def provide_queryset(self):
		return ISOCountry.objects.all()

class CountryMultipleChoiceField(_CustomChoiceField, forms.ModelMultipleChoiceField):
	"""A field for choosing multiple countries."""
	def provide_queryset(self):
		return ISOCountry.objects.all()
